@@ -29,7 +29,7 @@ def secrets(ctx: click.Context) -> None:
     pass
 
 
-def repeat_if_pin_needed(func) -> Callable:  # type: ignore[no-untyped-def]
+def repeat_if_pin_needed(func) -> Callable:  # type: ignore[no-untyped-def, type-arg]
     """
     Repeat the call of the decorated function, if PIN is required.
     Decorated function should have at least one argument,
@@ -74,6 +74,104 @@ def repeat_if_pin_needed(func) -> Callable:  # type: ignore[no-untyped-def]
     return wrapper
 
 
+@secrets.command()
+@click.pass_obj
+@click.argument(
+    "name",
+    type=click.STRING,
+)
+@click.argument(
+    "new_name",
+    type=click.STRING,
+)
+def rename(
+    ctx: Context,
+    name: str,
+    new_name: str,
+) -> None:
+    """
+    Rename credential.
+    """
+    with ctx.connect_device() as device:
+        app = SecretsApp(device)
+        ask_to_touch_if_needed()
+
+        @repeat_if_pin_needed
+        def call(app: SecretsApp) -> None:
+            app.rename_credential(
+                name.encode(),
+                new_name.encode(),
+            )
+
+        call(app)
+        local_print("Done")
+
+
+@secrets.command()
+@click.pass_obj
+@click.argument(
+    "name",
+    type=click.STRING,
+)
+@click.option(
+    "--login",
+    "login",
+    type=click.STRING,
+    help="Password Safe Login",
+    default=None,
+)
+@click.option(
+    "--password",
+    "password",
+    type=click.STRING,
+    help="Password Safe Password",
+    default=None,
+)
+@click.option(
+    "--metadata",
+    "metadata",
+    type=click.STRING,
+    help="Password Safe Metadata - additional field, to which extra information can be encoded",
+    default=None,
+)
+@click.option(
+    "--touch-button",
+    "touch_button",
+    type=click.BOOL,
+    help="Activate/deactivate touch button requirement",
+    default=None,
+)
+def update(
+    ctx: Context,
+    name: str,
+    new_name: Optional[bytes] = None,
+    login: Optional[bytes] = None,
+    password: Optional[bytes] = None,
+    metadata: Optional[bytes] = None,
+    touch_button: Optional[bool] = None,
+) -> None:
+    """
+    Update credential. Change Static Password fields, or touch button requirement attribute.
+    """
+    with ctx.connect_device() as device:
+        app = SecretsApp(device)
+        ask_to_touch_if_needed()
+
+        @repeat_if_pin_needed
+        def call(app: SecretsApp) -> None:
+            app.update_credential(
+                name.encode(),
+                new_name=new_name,
+                login=login,
+                password=password,
+                metadata=metadata,
+                touch_button=touch_button,
+            )
+
+        call(app)
+        local_print("Done")
+
+
 @secrets.command(aliases=["register"])
 @click.pass_obj
 @click.argument(
@@ -137,9 +235,9 @@ def add_otp(
     touch_button: bool,
     pin_protection: bool,
 ) -> None:
-    """Register OTP Credential.
+    """Register OTP credential.
 
-    Write Credential under the NAME.
+    Write credential under the NAME.
     Secret should be base32 encoded.
     """
     otp_kind = STRING_TO_KIND[kind.upper()]
@@ -209,7 +307,7 @@ def add_otp(
     "--metadata",
     "metadata",
     type=click.STRING,
-    help="Password Safe Metadata - additional field, to which extra information can be encoded in the future",
+    help="Password Safe Metadata - additional field, to which extra information can be encoded",
     default=None,
 )
 def add_password(
@@ -221,9 +319,9 @@ def add_password(
     password: Optional[bytes] = None,
     metadata: Optional[bytes] = None,
 ) -> None:
-    """Register Password Safe Credential.
+    """Register Password Safe credential.
 
-    Write Credential under the NAME.
+    Write credential under the NAME.
 
     """
 
@@ -258,7 +356,7 @@ def add_password(
     type=click.STRING,
 )
 def add_challenge_response(ctx: Context, slot: str, secret: str) -> None:
-    """Register Challenge-Response Credential."""
+    """Register Challenge-Response credential."""
 
     secret_bytes = b32decode(secret)
     sl = len(secret_bytes)
